function [Iout, Ioutrnd, M] = randKaczmarz(name, NumOfIter, NumPP)

% procedure for experimenting with random Kaczmarz algorithm in binary
% tomography application
% name = name of input file
% NumOfIter = number of iteration
% NumPP = number of projection planes

 I = imread(name);
 maxI = max(max(I));
 minI = min(min(I));
 I = I > (maxI + minI)/2;
 
 map = [ (0:1/200:1)' (0:1/200:1)' (0:1/200:1)' ];


% Image size (rows x columns).
imageSize = size(I);
%Center of image
imC = [floor(imageSize/2) + 1];
%Distance betwen the center and the border of the image
imL = floor(imageSize(2)/2);
% Numer of projections
%NumPP = 10;
% Angles for which the projection is calculated.
ang = 180 / NumPP;
angles = [0 : ang : (180 - ang)];
%Number of projection planes
numPP = length (angles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make matirces for solving linear equation system %
% Make B vector (vector of all projections).       %
% Make projection matrices for all the projections %
% and put them in the one big matrix.              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
matA = [];
matb = [];
sizeProj = [];
for i = 1 : numPP
    [tmpA, tmpb]=projectionmatrix(I, angles(i)*pi/180, imC, imL);
    
    matA = [matA; tmpA];
    matb = [matb; tmpb];
    sizeProj = [sizeProj length(tmpb)];
    allMatA{i} = tmpA;
end
%for

%Number of projections
numProj = length(matb);
%Number of pixels
numPix = imageSize(1) * imageSize(2);
% Quadratic Euclid norm of matrice A by rows.
QENA = ( sum( (matA').^2 ) );
if ( 0 < sum(QENA <= 0) )
    disp('ERROR!!!!')
end
%if
%Probability density function
SQ = sum(QENA);
PQ = QENA/SQ;
PQ = sort(PQ, 'descend');
for (i = 2 : length(PQ))
    PQ(i) = PQ(i-1) + PQ(i);
end
%for

%Lambda: weight of error impact on result
lambda = 1 + numProj/numPix;

% Matrix with values of all pixels in the image (in vector form).
% Pixel value are initialized to zeros.
x = zeros( numPix , 1 );

%Initial mask (mask is used to band calculation of some indexes that 
%have very low value)
mask = ones( numPix , 1 );

% Choose between random and sequential solver of the linear equation system
random = 1;

lastERR = inf;
DiffERR = inf;
% Make iterations
for (i = 1 : NumOfIter)
    
    j = 0;
    if (0 == random)
        %Go trough all projections sequentially
        j = mod(i, numProj) + 1;        
    else
        %Random choose projection index
        rnd = rand(1,1);
        PQ;
        for (r = 1 : length(PQ))
            if (r == 1)
                first = 0;
            else
                first = PQ(r-1);
            end
            last = PQ(r);
            if ( (first <= rnd) & (last >= rnd) )
                j = r;
            end
        end
        %for
    end
    %if
    if (j == 0)
        disp('Error!!');
    end
    %for
    %problem with projection matrix
    if (QENA(j) == 0)
        j = mod(i+1, numProj) + 1;  
    end
    %if
    
    % Calculate the error.
    Error = matb(j) - matA(j, :) * x;

    %Calculate the pixels incremental value
    dx = lambda.*(Error ./ QENA(j)) .* matA(j, :).';
    
    %Calculate new pixel values of reconstructed image.
    %Iteratively find the best dx that will not go outside 
    %of boundaries and will preserve the energy
    while(0 < sum(abs(dx)))
        x = x + dx;
        Ip = find(x > 1);
        In = find(x < 0);
        Iok = find( (0 <= x) & (1 >= x) );
        Nok = length(Iok);
        err = sum(x(In)) + sum(1 - x(Ip));
        %clipp x
        x(In) = 0;
        x(Ip) = 1;
        %new dx
        dx = zeros(numPix, 1);
        if (Nok > 0)
            dx(Iok) = err/Nok;
        end
        %if
    end
    %while
    
    %Mask image
    x = x .* mask;
    
    %Calculate the whole average error
    newERR = ( sum( abs(matb - matA * x) ) ) / numProj;
    
    %Difference between errors in two last iterations
    DiffERR = abs(lastERR - newERR);
        
    %Every 10 iterations ckeck the error
    if ( 10 == mod(i, 11) )
        %Save the error for the next check;
        lastERR = newERR;
        
        %If the error is small enough round the values!
        if (0.01 > abs(DiffERR))
            %normalize the pixels
            xmax = max(x);
            xmin = min(x);
            x = (x - xmin) ./ (xmax - xmin);
            %calculate the new mask
            Mask = x > 0.2;
            
            %x = x > 0.5;
        end
        %if
    end
    %if

    disp([ 'Iter: ' num2str(i) ', Projection: ' num2str(j) ...
           ', Error: ' num2str(newERR) ', DiffError: ' num2str(DiffERR) ]);
       
           
    %print reconstructed image
%     Iout = vec2mat(x, size(I, 2) )';
%     
%     matbrec = [];
%     for i = 1 : numPP
%         [tmpA, tmpb]=projectionmatrix( Iout , angles(i)*pi/180, imC, imL);
% 
%         matbrec = [matbrec; tmpb];
%     end
%     %for
% 
%     %reverse vector for ploting
%     for i = 1:numProj
%         matbrecr(numProj - i + 1) = matbrec(i);
%     end
%     %for
% 
%     %print each iteration step
%     subplot(2,2,1); imagesc(Iout); colormap ('gray'); colorbar
%     subplot(2,2,3); bar ( matbrec ( (numProj/2 + 1) : numProj ) ); title('90 deg Projection');
%     subplot(2,2,2); barh( matbrecr( (numProj/2 + 1) : numProj ) ); title('0 deg Projection');
    
    %input('');
    
    Iout = vec2mat(x, size(I, 2) )';
    Iout = double(round(Iout * 200) + 1);
    M(i) = im2frame(Iout, map);
    
end 
%for i = 1 : NumOfIter

Iout = vec2mat(x, size(I, 2) )';


%%%%%%%%%%%%%%%%%%%%%%%%
% Printing the results %
%%%%%%%%%%%%%%%%%%%%%%%%

%reverse vector for ploting
for i = 1:numProj
    matbr(numProj - i + 1) = matb(i);
end
%for

%print original image
figure;
subplot(2,4,1); imagesc(I); title('original image'); colormap ('gray'); colorbar
subplot(2,4,5); bar ( matbr( (numProj/2 + 1) : numProj   ) ); title('90 deg Projection');
subplot(2,4,2); barh( matb (               1 : numProj/2) ); title('0 deg Projection');

%print reconstructed image
matbrec = [];
for i = 1 : numPP
    [tmpA, tmpb]=projectionmatrix( Iout > 0.5 , angles(i)*pi/180, imC, imL);
    
    matbrec = [matbrec; tmpb];
end
%for

%reverse vector for ploting
for i = 1:numProj
    matbrecr(numProj - i + 1) = matbrec(i);
end
%for

subplot(2,4,3); imagesc(Iout); title('Reconstructed image'); colormap ('gray'); colorbar
subplot(2,4,7); bar ( matbrecr( (numProj/2 + 1) : numProj  ) ); title('90 deg Projection');
subplot(2,4,4); barh( matbrec (               1 : numProj/2) ); title('0 deg Projection');
subplot(2,4,8); imagesc(Iout>0.5); title('Reconstructed image');


Ioutrnd = Iout > .5;


movie2avi(M, 'movie1.avi');





